@@ -23,6 +23,7 @@ fn main() {
     if env::var("DOCS_RS").is_ok() {
         println!("cargo:warning=docs.rs detected - using stub bindings");
         println!("cargo:rustc-cfg=php_debug");
+        println!("cargo:rustc-cfg=php81");
 
         std::fs::copy("docsrs_bindings.rs", out_path)
             .expect("Unable to copy docs.rs stub bindings to output directory.");
@@ -101,7 +102,7 @@ fn main() {
         .clang_args(includes.split(' '))
         .parse_callbacks(Box::new(bindgen::CargoCallbacks))
         .rustfmt_bindings(true)
-        .no_copy("_zend_value")
+        .no_copy("_zval_struct")
         .no_copy("_zend_string")
         .no_copy("_zend_array")
         .layout_tests(env::var("EXT_PHP_RS_TEST").is_ok());
