@@ -394,6 +394,60 @@ impl ZendHashTable {
         Ok(())
     }
 
+    /// Checks if the hashtable only contains numerical keys.
+    ///
+    /// # Returns
+    ///
+    /// True if all keys on the hashtable are numerical.
+    ///
+    /// # Example
+    ///
+    /// ```no_run
+    /// use ext_php_rs::types::ZendHashTable;
+    ///
+    /// let mut ht = ZendHashTable::new();
+    ///
+    /// ht.push(0);
+    /// ht.push(3);
+    /// ht.push(9);
+    /// assert!(ht.has_numerical_keys());
+    ///
+    /// ht.insert("obviously not numerical", 10);
+    /// assert!(!ht.has_numerical_keys());
+    /// ```
+    pub fn has_numerical_keys(&self) -> bool {
+        !self.iter().any(|(_, k, _)| k.is_some())
+    }
+
+    /// Checks if the hashtable has numerical, sequential keys.
+    ///
+    /// # Returns
+    ///
+    /// True if all keys on the hashtable are numerical and are in sequential
+    /// order (i.e. starting at 0 and not skipping any keys).
+    ///
+    /// # Example
+    ///
+    /// ```no_run
+    /// use ext_php_rs::types::ZendHashTable;
+    ///
+    /// let mut ht = ZendHashTable::new();
+    ///
+    /// ht.push(0);
+    /// ht.push(3);
+    /// ht.push(9);
+    /// assert!(ht.has_sequential_keys());
+    ///
+    /// ht.insert_at_index(90, 10);
+    /// assert!(!ht.has_sequential_keys());
+    /// ```
+    pub fn has_sequential_keys(&self) -> bool {
+        !self
+            .iter()
+            .enumerate()
+            .any(|(i, (k, strk, _))| i as u64 != k || strk.is_some())
+    }
+
     /// Returns an iterator over the key(s) and value contained inside the
     /// hashtable.
     ///
@@ -616,13 +670,13 @@ impl<'a> FromZval<'a> for &'a ZendHashTable {
 //// HashMap
 ///////////////////////////////////////////
 
-impl<V> TryFrom<&ZendHashTable> for HashMap<String, V>
+impl<'a, V> TryFrom<&'a ZendHashTable> for HashMap<String, V>
 where
-    for<'a> V: FromZval<'a>,
+    V: FromZval<'a>,
 {
     type Error = Error;
 
-    fn try_from(value: &ZendHashTable) -> Result<Self> {
+    fn try_from(value: &'a ZendHashTable) -> Result<Self> {
         let mut hm = HashMap::with_capacity(value.len());
 
         for (idx, key, val) in value.iter() {
@@ -670,13 +724,13 @@ where
     }
 }
 
-impl<T> FromZval<'_> for HashMap<String, T>
+impl<'a, T> FromZval<'a> for HashMap<String, T>
 where
-    for<'a> T: FromZval<'a>,
+    T: FromZval<'a>,
 {
     const TYPE: DataType = DataType::Array;
 
-    fn from_zval(zval: &Zval) -> Option<Self> {
+    fn from_zval(zval: &'a Zval) -> Option<Self> {
         zval.array().and_then(|arr| arr.try_into().ok())
     }
 }
@@ -685,13 +739,13 @@ where
 //// Vec
 ///////////////////////////////////////////
 
-impl<T> TryFrom<&ZendHashTable> for Vec<T>
+impl<'a, T> TryFrom<&'a ZendHashTable> for Vec<T>
 where
-    for<'a> T: FromZval<'a>,
+    T: FromZval<'a>,
 {
     type Error = Error;
 
-    fn try_from(value: &ZendHashTable) -> Result<Self> {
+    fn try_from(value: &'a ZendHashTable) -> Result<Self> {
         let mut vec = Vec::with_capacity(value.len());
 
         for (_, _, val) in value.iter() {
@@ -734,13 +788,13 @@ where
     }
 }
 
-impl<T> FromZval<'_> for Vec<T>
+impl<'a, T> FromZval<'a> for Vec<T>
 where
-    for<'a> T: FromZval<'a>,
+    T: FromZval<'a>,
 {
     const TYPE: DataType = DataType::Array;
 
-    fn from_zval(zval: &Zval) -> Option<Self> {
+    fn from_zval(zval: &'a Zval) -> Option<Self> {
         zval.array().and_then(|arr| arr.try_into().ok())
     }
 }
