@@ -7,7 +7,7 @@ use std::{convert::TryInto, ffi::c_void, fmt::Debug, ptr};
 use crate::{
     binary::Pack,
     boxed::ZBox,
-    convert::{FromZval, IntoZval, IntoZvalDyn},
+    convert::{FromZval, FromZvalMut, IntoZval, IntoZvalDyn},
     error::{Error, Result},
     ffi::{
         _zval_struct__bindgen_ty_1, _zval_struct__bindgen_ty_2, zend_is_callable, zend_resource,
@@ -259,7 +259,7 @@ impl Zval {
 
     /// Returns true if the zval is true, false otherwise.
     pub fn is_true(&self) -> bool {
-        self.get_type() == DataType::False
+        self.get_type() == DataType::True
     }
 
     /// Returns true if the zval is false, false otherwise.
@@ -500,6 +500,38 @@ impl Zval {
     {
         FromZval::from_zval(self)
     }
+
+    /// Creates a shallow clone of the [`Zval`].
+    ///
+    /// This copies the contents of the [`Zval`], and increments the reference
+    /// counter of the underlying value (if it is reference counted).
+    ///
+    /// For example, if the zval contains a long, it will simply copy the value.
+    /// However, if the zval contains an object, the new zval will point to the
+    /// same object, and the objects reference counter will be incremented.
+    ///
+    /// # Returns
+    ///
+    /// The cloned zval.
+    pub fn shallow_clone(&self) -> Zval {
+        let mut new = Zval::new();
+        new.u1 = self.u1;
+        new.value = self.value;
+
+        // SAFETY: `u1` union is only used for easier bitmasking. It is valid to read
+        // from either of the variants.
+        //
+        // SAFETY: If the value if refcounted (`self.u1.type_info & Z_TYPE_FLAGS_MASK`)
+        // then it is valid to dereference `self.value.counted`.
+        unsafe {
+            let flags = ZvalTypeFlags::from_bits_unchecked(self.u1.type_info);
+            if flags.contains(ZvalTypeFlags::RefCounted) {
+                (*self.value.counted).gc.refcount += 1;
+            }
+        }
+
+        new
+    }
 }
 
 impl Debug for Zval {
@@ -558,3 +590,19 @@ impl IntoZval for Zval {
         Ok(())
     }
 }
+
+impl<'a> FromZval<'a> for &'a Zval {
+    const TYPE: DataType = DataType::Mixed;
+
+    fn from_zval(zval: &'a Zval) -> Option<Self> {
+        Some(zval)
+    }
+}
+
+impl<'a> FromZvalMut<'a> for &'a mut Zval {
+    const TYPE: DataType = DataType::Mixed;
+
+    fn from_zval_mut(zval: &'a mut Zval) -> Option<Self> {
+        Some(zval)
+    }
+}
